@@ -14,16 +14,12 @@
 // app includes
 #include "association.h"
 #include "artifact.h"
-#include "attribute.h"
 #include "classifier.h"
+#include "datatype.h"
 #include "debug_utils.h"
 #include "folder.h"
 #include "enum.h"
-#include "enumliteral.h"
-#include "messagewidget.h"
-#include "model_utils.h"
 #include "object_factory.h"
-#include "objectwidget.h"
 #include "operation.h"
 #include "package.h"
 #include "template.h"
@@ -31,17 +27,11 @@
 #include "umldoc.h"
 #include "umllistview.h"
 #include "umlobject.h"
-#include "umbrellosettings.h"
-#include "widget_factory.h"
 
 // kde includes
 #include <KMessageBox>
 #include <KLocalizedString>
 
-// qt includes
-#include <QMap>
-#include <QRegExp>
-
 #ifdef Q_OS_WIN
 #define PATH_SEPARATOR QLatin1Char(';')
 #else
@@ -66,7 +56,7 @@ bool bNewUMLObjectWasCreated = false;
  * Related classifier for creation of dependencies on template
  * parameters in createUMLObject().
  */
-UMLClassifier * gRelatedClassifier = NULL;
+UMLClassifier * gRelatedClassifier = 0;
 
 /**
  * On encountering a scoped typename string where the scopes
@@ -187,9 +177,9 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
     QString name = inName;
     UMLDoc *umldoc = UMLApp::app()->document();
     UMLFolder *logicalView = umldoc->rootFolder(Uml::ModelType::Logical);
-    if (parentPkg == NULL) {
+    if (parentPkg == 0) {
         // DEBUG(DBG_SRC) << "Import_Utils::createUMLObject(" << name
-        //     << "): parentPkg is NULL, assuming Logical View";
+        //     << "): parentPkg is 0, assuming Logical View";
         parentPkg = logicalView;
     } else if (parentPkg->baseType() == UMLObject::ot_Artifact) {
         DEBUG(DBG_SRC) << "Import_Utils::createUMLObject(" << name
@@ -199,7 +189,7 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
         DEBUG(DBG_SRC) << "Import_Utils::createUMLObject(" << name
                        << "): Association as parent package is not supported yet, using Logical View";
         parentPkg = logicalView;
-    } else if (name.startsWith(QLatin1String("::"))) {
+    } else if (name.startsWith(UMLApp::app()->activeLanguageScopeSeparator())) {
         name = name.mid(2);
         parentPkg = logicalView;
     }
@@ -212,7 +202,7 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
             bPutAtGlobalScope = false;
         }
     }
-    if (o == NULL) {
+    if (o == 0) {
         // Strip possible adornments and look again.
         const bool isConst = name.contains(QRegExp(QLatin1String("^const ")));
         name.remove(QRegExp(QLatin1String("^const\\s+")));
@@ -223,21 +213,20 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
         typeName.remove(QRegExp(QLatin1String("[^\\w:\\. ].*$")));
         typeName = typeName.simplified();
         UMLObject *origType = umldoc->findUMLObject(typeName, UMLObject::ot_UMLObject, parentPkg);
-        if (origType == NULL) {
+        if (origType == 0) {
             // Still not found. Create the stripped down type.
             if (bPutAtGlobalScope)
                 parentPkg = logicalView;
             // Find, or create, the scopes.
             QStringList components;
-            if (typeName.contains(QLatin1String("::"))) {
-                components = typeName.split(QLatin1String("::"), QString::SkipEmptyParts);
+            QString scopeSeparator = UMLApp::app()->activeLanguageScopeSeparator();
+            if (typeName.contains(scopeSeparator)) {
+                components = typeName.split(scopeSeparator, QString::SkipEmptyParts);
             } else if (typeName.contains(QLatin1String("..."))) {
                 // Java variable length arguments
                 type = UMLObject::ot_Datatype;
                 parentPkg = umldoc->datatypeFolder();
                 isAdorned = false;
-            } else if (typeName.contains(QLatin1String("."))) {
-                components = typeName.split(QLatin1Char('.'));
             }
             if (components.count() > 1) {
                 typeName = components.back();
@@ -247,15 +236,18 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
                     components.pop_front();
                     o = umldoc->findUMLObject(scopeName, UMLObject::ot_UMLObject, parentPkg);
                     if (o) {
-                        parentPkg = static_cast<UMLPackage*>(o);
+                        parentPkg = o->asUMLPackage();
                         continue;
                     }
                     o = Object_Factory::createUMLObject(UMLObject::ot_Class, scopeName, parentPkg);
                     o->setStereotypeCmd(QLatin1String("class-or-package"));
-                    UMLListViewItem *item = UMLApp::app()->listView()->findUMLObject(o);
-                    if (item)
-                        item->updateObject();
-                    parentPkg = static_cast<UMLPackage*>(o);
+                    // setStereotypeCmd() triggers tree view item update if not loading by default
+                    if (umldoc->loading()) {
+                        UMLListViewItem *item = UMLApp::app()->listView()->findUMLObject(o);
+                        if (item)
+                            item->updateObject();
+                    }
+                    parentPkg = o->asUMLPackage();
                     Model_Utils::treeViewSetCurrentItem(o);
                 }
                 // All scope qualified datatypes live in the global scope.
@@ -275,19 +267,19 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
             o = Object_Factory::createUMLObject(UMLObject::ot_Datatype, name,
                                                 umldoc->datatypeFolder(),
                                                 false); //solicitNewName
-            UMLClassifier *dt = static_cast<UMLClassifier*>(o);
-            UMLClassifier *c = dynamic_cast<UMLClassifier*>(origType);
-            if (c)
+            UMLDatatype *dt = o ? o->asUMLDatatype() : 0;
+            UMLClassifier *c = origType->asUMLClassifier();
+            if (dt && c)
                 dt->setOriginType(c);
             else
                 uError() << "createUMLObject(" << name << "): "
                     << "origType " << typeName << " is not a UMLClassifier";
-            if (isRef || isPointer)
+            if (dt && (isRef || isPointer))
                 dt->setIsReference();
             /*
             if (isPointer) {
                 UMLObject *pointerDecl = Object_Factory::createUMLObject(UMLObject::ot_Datatype, type);
-                UMLClassifier *dt = static_cast<UMLClassifier*>(pointerDecl);
+                UMLClassifier *dt = pointerDecl->asUMLClassifier();
                 dt->setOriginType(classifier);
                 dt->setIsReference();
                 classifier = dt;
@@ -297,24 +289,30 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
         }
     } else if (parentPkg && !bPutAtGlobalScope) {
         UMLPackage *existingPkg = o->umlPackage();
-        if (existingPkg != umldoc->datatypeFolder()) {
+        if (existingPkg != parentPkg && existingPkg != umldoc->datatypeFolder()) {
             if (existingPkg)
                 existingPkg->removeObject(o);
             else
                 uError() << "createUMLObject(" << name << "): "
                     << "o->getUMLPackage() was NULL";
-            o->setUMLPackage(parentPkg);
             parentPkg->addObject(o);
+            o->setUMLPackage(parentPkg);
+            // setUMLPackage() triggers tree view item update if not loading by default
+            if (umldoc->loading()) {
+                UMLListViewItem *item = UMLApp::app()->listView()->findUMLObject(o);
+                if (item)
+                    item->updateObject();
+            }
         }
     }
     QString strippedComment = formatComment(comment);
     if (! strippedComment.isEmpty()) {
         o->setDoc(strippedComment);
     }
-    if (!stereotype.isEmpty()) {
+    if (o && !stereotype.isEmpty()) {
         o->setStereotype(stereotype);
     }
-    if (gRelatedClassifier == NULL || gRelatedClassifier == o)
+    if (gRelatedClassifier == 0 || gRelatedClassifier == o)
         return o;
     QRegExp templateInstantiation(QLatin1String("^[\\w:\\.]+\\s*<(.*)>"));
     int pos = templateInstantiation.indexIn(name);
@@ -328,7 +326,7 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
     QStringList::ConstIterator end(params.end());
     for (QStringList::ConstIterator it(params.begin()); it != end; ++it) {
         UMLObject *p = umldoc->findUMLObject(*it, UMLObject::ot_UMLObject, parentPkg);
-        if (p == NULL || p->baseType() == UMLObject::ot_Datatype)
+        if (p == 0 || p->isUMLDatatype())
             continue;
         const Uml::AssociationType::Enum at = Uml::AssociationType::Dependency;
         UMLAssociation *assoc = umldoc->findAssociation(at, gRelatedClassifier, p);
@@ -338,7 +336,7 @@ UMLObject *createUMLObject(UMLObject::ObjectType type,
         assoc->setUMLPackage(umldoc->rootFolder(Uml::ModelType::Logical));
         umldoc->addAssociation(assoc);
     }
-    if (o == NULL) {
+    if (o == 0) {
         uError() << "is NULL!";
     }
     return o;
@@ -376,7 +374,7 @@ UMLObject* insertAttribute(UMLClassifier *owner,
            (ot == UMLObject::ot_Interface && pl == Uml::ProgrammingLanguage::Java))) {
         DEBUG(DBG_SRC) << "insertAttribute: Don not know what to do with "
                  << owner->name() << " (object type " << UMLObject::toString(ot) << ")";
-        return NULL;
+        return 0;
     }
     UMLObject *o = owner->findChildObject(name, UMLObject::ot_Attribute);
     if (o) {
@@ -404,15 +402,15 @@ UMLObject* insertAttribute(UMLClassifier *owner, Uml::Visibility::Enum scope,
                            bool isStatic /* =false */)
 {
     UMLObject *attrType = owner->findTemplate(type);
-    if (attrType == NULL) {
+    if (attrType == 0) {
         bPutAtGlobalScope = true;
         gRelatedClassifier = owner;
         attrType = createUMLObject(UMLObject::ot_UMLObject, type, owner);
-        gRelatedClassifier = NULL;
+        gRelatedClassifier = 0;
         bPutAtGlobalScope = false;
     }
     return insertAttribute (owner, scope, name,
-                            static_cast<UMLClassifier*>(attrType),
+                            attrType->asUMLClassifier(),
                             comment, isStatic);
 }
 
@@ -432,13 +430,14 @@ UMLObject* insertAttribute(UMLClassifier *owner, Uml::Visibility::Enum scope,
   * @param isAbstract     boolean switch to decide if method is abstract
   * @param isFriend       true boolean switch to decide if methods is a friend function
   * @param isConstructor  boolean switch to decide if methods is a constructor
+  * @param isDestructor   boolean switch to decide if methods is a destructor
   * @param comment        The Documentation for this method
   */
 void insertMethod(UMLClassifier *klass, UMLOperation* &op,
                   Uml::Visibility::Enum scope, const QString& type,
                   bool isStatic, bool isAbstract,
                   bool isFriend, bool isConstructor,
-                  const QString& comment)
+                  bool isDestructor, const QString& comment)
 {
     op->setVisibilityCmd(scope);
     if (!type.isEmpty()     // return type may be missing (constructor/destructor)
@@ -447,11 +446,11 @@ void insertMethod(UMLClassifier *klass, UMLOperation* &op,
             op->setType(klass);
         } else {
             UMLObject *typeObj = klass->findTemplate(type);
-            if (typeObj == NULL) {
+            if (typeObj == 0) {
                 bPutAtGlobalScope = true;
                 gRelatedClassifier = klass;
                 typeObj = createUMLObject(UMLObject::ot_UMLObject, type, klass);
-                gRelatedClassifier = NULL;
+                gRelatedClassifier = 0;
                 bPutAtGlobalScope = false;
                 op->setType(typeObj);
             }
@@ -467,6 +466,8 @@ void insertMethod(UMLClassifier *klass, UMLOperation* &op,
     // if the operation is a constructor, add it as a stereotype
     if (isConstructor)
         op->setStereotype(QLatin1String("constructor"));
+    if (isDestructor)
+        op->setStereotype(QLatin1String("destructor"));
 
     QString strippedComment = formatComment(comment);
     if (! strippedComment.isEmpty()) {
@@ -510,13 +511,13 @@ UMLAttribute* addMethodParameter(UMLOperation *method,
                                  const QString& type,
                                  const QString& name)
 {
-    UMLClassifier *owner = static_cast<UMLClassifier*>(method->parent());
-    UMLObject *typeObj = owner->findTemplate(type);
-    if (typeObj == NULL) {
+    UMLClassifier *owner = method->umlParent()->asUMLClassifier();
+    UMLObject *typeObj = owner ? owner->findTemplate(type) : 0;
+    if (typeObj == 0) {
         bPutAtGlobalScope = true;
         gRelatedClassifier = owner;
         typeObj = createUMLObject(UMLObject::ot_UMLObject, type, owner);
-        gRelatedClassifier = NULL;
+        gRelatedClassifier = 0;
         bPutAtGlobalScope = false;
     }
     UMLAttribute *attr = Object_Factory::createAttribute(method, name, typeObj);
@@ -537,7 +538,7 @@ void addEnumLiteral(UMLEnum *enumType, const QString &literal, const QString &co
  * Create a generalization from the given child classifier to the given
  * parent classifier.
  */
-void createGeneralization(UMLClassifier *child, UMLClassifier *parent)
+UMLAssociation *createGeneralization(UMLClassifier *child, UMLClassifier *parent)
 {
     // if the child is an interface, so is the parent.
     if (child->isInterface())
@@ -554,28 +555,72 @@ void createGeneralization(UMLClassifier *child, UMLClassifier *parent)
     UMLDoc *umldoc = UMLApp::app()->document();
     assoc->setUMLPackage(umldoc->rootFolder(Uml::ModelType::Logical));
     umldoc->addAssociation(assoc);
+    return assoc;
+}
+
+/**
+ * Create a subdir with the given name.
+ */
+UMLFolder *createSubDir(const QString& name,
+                        UMLFolder *parentPkg,
+                        const QString &comment)
+{
+    UMLDoc *umldoc = UMLApp::app()->document();
+    if (!parentPkg) {
+        parentPkg = umldoc->rootFolder(Uml::ModelType::Component);
+    }
+
+    UMLObject::ObjectType type = UMLObject::ot_Folder;
+    UMLFolder *o = umldoc->findUMLObjectRaw(parentPkg, name, type)->asUMLFolder();
+    if (o)
+        return o;
+    o = Object_Factory::createUMLObject(type, name, parentPkg, false)->asUMLFolder();
+    if (o)
+        o->setDoc(comment);
+    return o;
+}
+
+
+/**
+  * Create a folder for artifacts
+ */
+UMLObject *createArtifactFolder(const QString& name,
+                                UMLPackage *parentPkg,
+                                const QString &comment)
+{
+    Q_UNUSED(parentPkg);
+
+    UMLObject::ObjectType type = UMLObject::ot_Folder;
+    UMLDoc *umldoc = UMLApp::app()->document();
+    UMLFolder *componentView = umldoc->rootFolder(Uml::ModelType::Component);
+    UMLObject *o = umldoc->findUMLObjectRaw(componentView, name, type);
+    if (o)
+        return o;
+    o = Object_Factory::createUMLObject(type, name, componentView, false);
+    UMLFolder *a = o->asUMLFolder();
+    a->setDoc(comment);
+    DEBUG(DBG_SRC) << name << comment;
+    return o;
 }
 
 /**
  * Create an artifact with the given name.
  */
 UMLObject *createArtifact(const QString& name,
-                          UMLPackage *parentPkg,
+                          UMLFolder *parentPkg,
                           const QString &comment)
 {
-    Q_UNUSED(parentPkg);
+    UMLDoc *umldoc = UMLApp::app()->document();
+    if (!parentPkg) {
+        parentPkg = umldoc->rootFolder(Uml::ModelType::Component);
+    }
 
     UMLObject::ObjectType type = UMLObject::ot_Artifact;
-    if (!Settings::optionState().codeImportState.createArtifacts)
-        return 0;
-    UMLDoc *umldoc = UMLApp::app()->document();
-    UMLFolder *componentView = umldoc->rootFolder(Uml::ModelType::Component);
-    QFileInfo fi(name);
-    UMLObject *o = umldoc->findUMLObjectRaw(componentView, fi.fileName(), type);
+    UMLObject *o = umldoc->findUMLObjectRaw(parentPkg, name, type);
     if (o)
         return o;
-    o = Object_Factory::createUMLObject(type, fi.fileName(), componentView, false);
-    UMLArtifact *a = static_cast<UMLArtifact*>(o);
+    o = Object_Factory::createUMLObject(type, name, parentPkg, false);
+    UMLArtifact *a = o->asUMLArtifact();
     a->setDrawAsType(UMLArtifact::file);
     a->setDoc(comment);
     DEBUG(DBG_SRC) << name << comment;
@@ -593,10 +638,44 @@ UMLObject *createArtifact(const QString& name,
 void createGeneralization(UMLClassifier *child, const QString &parentName)
 {
     UMLObject *parentObj = createUMLObject(UMLObject::ot_Class, parentName);
-    UMLClassifier *parent = static_cast<UMLClassifier*>(parentObj);
+    UMLClassifier *parent = parentObj->asUMLClassifier();
     createGeneralization(child, parent);
 }
 
+/**
+ * Remap UMLObject instance in case it does not have the correct type.
+ *
+ * @param ns uml object instance with incorrect class
+ * @param currentScope parent uml object
+ * @return newly created UMLEnum instance or zero in case of error
+ */
+UMLEnum *remapUMLEnum(UMLObject *ns, UMLPackage *currentScope)
+{
+    if (ns) {
+        QString comment = ns->doc();
+        QString name = ns->name();
+        QString stereotype = ns->stereotype();
+        Uml::Visibility::Enum visibility = ns->visibility();
+        UMLApp::app()->document()->removeUMLObject(ns, true);
+        if (currentScope == 0)
+            currentScope = UMLApp::app()->document()->rootFolder(Uml::ModelType::Logical);
+        UMLObject *o = Object_Factory::createNewUMLObject(UMLObject::ot_Enum, name, currentScope, false);
+        if (!o)
+            return 0;
+        UMLEnum *e = o->asUMLEnum();
+        if (!e)
+            return 0;
+        e->setDoc(comment);
+        e->setStereotypeCmd(stereotype.isEmpty() ? QLatin1String("enum") : stereotype);
+        e->setVisibilityCmd(visibility);
+        // add to parents child list
+        if (!currentScope->containedObjects().contains(e))
+            currentScope->containedObjects().append(e);
+        return e;
+    }
+    return 0;
+}
+
 /**
  * Return the list of paths set by previous calls to addIncludePath()
  * and the environment variable UMBRELLO_INCPATH.
@@ -629,235 +708,7 @@ bool isDatatype(const QString& name, UMLPackage *parentPkg)
 {
     UMLDoc *umldoc = UMLApp::app()->document();
     UMLObject * o = umldoc->findUMLObject(name, UMLObject::ot_Datatype, parentPkg);
-    return (o!=NULL);
-}
-
-/**
- * Helper function to parse stack trace line. The supported formats are:
- *
- * 1. gdb
- * 2. Qtcreator/gdb
- *
- * @param s string to parse
- * @param sequence return of sequence number
- * @param package return of package
- * @param method return of method
- * @return true line could be parsed and return variable has been filled
- * @return false line could not be parsed, no return variable has been filled
- */
-bool parseStraceTraceLine(const QString &s, QString &sequence, QString &package, QString &method)
-{
-    QString identifier;
-    QString module;
-    QStringList cols = s.split(QRegExp(QLatin1String("\\s+")),QString::SkipEmptyParts);
-    if (cols.size() < 2) {
-        DEBUG(DBG_SRC) << "could not parse" << s;
-        return false;
-    }
-    // skip comments
-    if (cols[0] == QLatin1String("#")) {
-        return false;
-    }
-
-    /*  gdb
-     * #0  0x000000000050d0b0 in Import_Utils::importStackTrace(QString const&, UMLScene*) (fileName=..., scene=scene@entry=0x12bd0f0)
-     * #25 0x00007fffefe1670c in g_main_context_iteration () from /usr/lib64/libglib-2.0.so.0
-     * #0  Import_Utils::importStackTrace (fileName=..., scene=scene@entry=0x137c000) at /home/ralf.habacker/src/umbrello-3/umbrello/codeimport/import_utils.cpp:715
-     */
-    if (cols.size() > 2 && cols[0].startsWith(QLatin1String("#"))) {
-        QString file;
-        sequence = cols.takeFirst();
-        if (cols[cols.size()-2] == QLatin1String("at")) {
-            file = cols.takeLast();
-            cols.takeLast();
-        }
-        else if (cols[cols.size()-2] == QLatin1String("from")) {
-            module = cols.takeLast();
-            cols.takeLast();
-        }
-
-        if (cols[1] == QLatin1String("in")) {
-            cols.takeFirst(); // remove address
-            cols.takeFirst(); // in
-        }
-
-        identifier = cols.join(QLatin1String(" "));
-
-        if (identifier.contains(QLatin1String("::"))) {
-            QStringList b = identifier.split( QLatin1String("::"));
-            // TODO handle multiple '::'
-            package = b.takeFirst();
-            method = b.join(QLatin1String("::"));
-        }
-        else {
-            method = identifier;
-            package = module;
-        }
-        return true;
-    }
-
-    /**
-     * Qtcreator/gdb
-     *  6	Driver::ParseHelper::ParseHelper	driver.cpp	299	0x634c44
-     * 31   g_main_context_dispatch /usr/lib64/libglib-2.0.so.0     0x7fffefe16316
-     */
-    else if (cols[cols.size()-1].startsWith(QLatin1String("0x"))) {
-        if (cols[0] == QLatin1String("..."))
-            return false;
-
-        sequence = cols.takeFirst();
-        cols.takeLast();  // remove address
-
-        QString line, file;
-        if (cols.size() == 2) {
-            module = cols.takeLast();
-            identifier = cols.join(QLatin1String(" "));
-        } else {
-            line = cols.takeLast();
-            file = cols.takeLast();
-            identifier = cols.join(QLatin1String(" "));
-        }
-
-        if (identifier.contains(QLatin1String("::"))) {
-            QStringList b = identifier.split( QLatin1String("::"));
-            method = b.takeLast();
-            package = b.join(QLatin1String("::"));
-        }
-        else {
-            method = identifier;
-            package = module;
-        }
-
-        if (package.isEmpty() && !file.isEmpty())
-            package = file;
-
-        if (!method.endsWith(QLatin1String(")")))
-            method.append(QLatin1String("()"));
-
-        return true;
-    } else
-        return false;
-}
-
-/**
- * Import stack trace generated by gdb or qtcreator running gdb.
- *
- * The stack trace can be generated by
- *  1. copy and paste of the output of gdb 'bt' command into a file (single line for each stack frame)
- *  2. select frames in qtcreator stack window, right click and select "copy into clipboard", then paste into file
- *
- * @param fileName filename to import the stack trace from.
- * @param scene The diagram to import the stack trace into.
- * @return true Import successful.
- * @return false Import failed.
- */
-bool importStackTrace(const QString &fileName, UMLScene *scene)
-{
-    QFile file(fileName);
-
-    if(!file.open(QIODevice::ReadOnly))
-        return false;
-
-    // reverse lines
-    QStringList lines;
-    QTextStream in(&file);
-    in.setCodec("UTF-8");
-    while (!in.atEnd()) {
-        lines.insert(0, in.readLine());
-    }
-
-    // object widget cache map
-    QMap<QString, ObjectWidget*> objectsMap;
-
-    // create "client widget"
-    UMLDoc *umldoc = UMLApp::app()->document();
-    QString name(QLatin1String("client"));
-    UMLObject *left = umldoc->findUMLObject(name, UMLObject::ot_Class);
-    if (!left ) {
-        left = new UMLObject(0, name);
-        left->setBaseType(UMLObject::ot_Class);
-    }
-
-    ObjectWidget *leftWidget = (ObjectWidget *)Widget_Factory::createWidget(scene, left);
-    leftWidget->activate();
-    // required to be savable
-    scene->widgetList().append(leftWidget);
-    objectsMap[name] = leftWidget;
-
-    ObjectWidget *rightWidget = 0;
-    ObjectWidget *mostRightWidget = leftWidget;
-    MessageWidget *messageWidget = 0;
-    // for further processing
-    MessageWidgetList messages;
-
-    // for each line
-    int index = 1;
-    foreach(const QString &line, lines) {
-        QString stackframe, package, method;
-
-        if (!parseStraceTraceLine(line, stackframe, package, method))
-            continue;
-
-        bool createObject = false;
-        if (package.contains(method))
-            createObject = true;
-
-        if (package.isEmpty())
-            package = QLatin1String("unknown");
-
-        // get or create right object widget
-        if (objectsMap.contains(package)) {
-            rightWidget = objectsMap[package];
-        } else {
-            UMLObject *right = umldoc->findUMLObject(package, UMLObject::ot_Class);
-            if (!right)
-                right = umldoc->findUMLObject(package, UMLObject::ot_Package);
-            if (!right) {
-                right = Object_Factory::createUMLObject(UMLObject::ot_Class, package);
-            }
-
-            rightWidget = (ObjectWidget *)Widget_Factory::createWidget(scene, right);
-            rightWidget->setX(mostRightWidget->x() + mostRightWidget->width() + 10);
-            rightWidget->activate();
-            objectsMap[package] = rightWidget;
-            scene->widgetList().append(rightWidget);
-            mostRightWidget = rightWidget;
-        }
-
-        // create message
-        int y = 10 + (messageWidget ? messageWidget->y() + messageWidget->height() : 20);
-        messageWidget = new MessageWidget(scene, leftWidget, rightWidget, y,
-                                          createObject ? Uml::SequenceMessage::Creation : Uml::SequenceMessage::Synchronous);
-        messageWidget->setCustomOpText(method);
-        messageWidget->setSequenceNumber(QString::number(index++));
-        messageWidget->calculateWidget();
-        messageWidget->activate();
-        messageWidget->setY(y);
-        // to make it savable
-        scene->messageList().append(messageWidget);
-        messages.insert(0, messageWidget);
-
-        leftWidget = rightWidget;
-    }
-
-    if (messages.isEmpty())
-        return false;
-
-    // adjust vertical position
-    foreach(MessageWidget *w, messages) {
-        w->setY(w->y() + 20);
-    }
-
-    // adjust heights starting from the last message
-    MessageWidget *previous = messages.takeFirst();
-    foreach(MessageWidget *w, messages) {
-        w->setSize(w->width(), previous->y() - w->y() + previous->height() + 5);
-        // adjust vertical line length of object widgets
-        w->objectWidget(Uml::RoleType::A)->slotMessageMoved();
-        w->objectWidget(Uml::RoleType::B)->slotMessageMoved();
-        previous = w;
-    }
-    return true;
+    return (o != 0);
 }
 
 }  // end namespace Import_Utils
